﻿// BluetoothDeviceManager.cpp : 구현 파일입니다.
//

#include "stdafx.h"
#include "MFC_BLUETOOTH_TEST.h"
#include "BluetoothDeviceManager.h"

#include <dbt.h>

CEvent g_callbackevent;

BOOL WINAPI auth_callback_ex(LPVOID pvParam, PBLUETOOTH_AUTHENTICATION_CALLBACK_PARAMS authParams)
{
	BLUETOOTH_AUTHENTICATE_RESPONSE response = { 0, };
	response.bthAddressRemote = authParams->deviceInfo.Address;
	response.authMethod = authParams->authenticationMethod;// authParams->authenticationMethod; // == BLUETOOTH_AUTHENTICATION_METHOD_LEGACY	

	UCHAR pin[] = "123456";
	std::copy(pin, pin + sizeof(pin), response.pinInfo.pin);
	response.pinInfo.pinLength = 6; //excluding '\0'

	response.negativeResponse = false;

	HRESULT err = BluetoothSendAuthenticationResponseEx(NULL, &response);

	if (err == S_OK)
	{
		CBluetoothDeviceManager *pCBluetoothDeviceManager = reinterpret_cast<CBluetoothDeviceManager*>(pvParam);

		if (pCBluetoothDeviceManager != NULL)
			pCBluetoothDeviceManager->SendPairing();
	}

	if (err)
	{
		std::wcout << "BluetoothSendAuthenticationResponseEx error = " << err << std::endl;
	}

	g_callbackevent.PulseEvent();

	return true;
}

// CBluetoothDeviceManager

CBluetoothDeviceManager::CBluetoothDeviceManager(const CBluetoothRadioManager &rCBluetoothRadioManager, BLUETOOTH_DEVICE_INFO_STRUCT &rBLUETOOTH_DEVICE_INFO_STRUCT) :
m_rCBluetoothRadioManager(rCBluetoothRadioManager),
m_BLUETOOTH_DEVICE_INFO_STRUCT(rBLUETOOTH_DEVICE_INFO_STRUCT),
m_hHBLUETOOTH_AUTHENTICATION_REGISTRATION(NULL)
{
	std::wcout << _T("Device found - Name : ") << m_BLUETOOTH_DEVICE_INFO_STRUCT.szName << std::endl;
	std::wcout << _T("Device found - Address : ") << m_BLUETOOTH_DEVICE_INFO_STRUCT.Address.rgBytes << std::endl;
	std::wcout << _T("Device found - Device Class : ") << m_BLUETOOTH_DEVICE_INFO_STRUCT.ulClassofDevice << std::endl;

	DWORD dwResult = BluetoothRegisterForAuthenticationEx(&m_BLUETOOTH_DEVICE_INFO_STRUCT, &m_hHBLUETOOTH_AUTHENTICATION_REGISTRATION, &auth_callback_ex, this);
	
	if (dwResult == ERROR_OUTOFMEMORY)
	{
		std::wcerr << _T("BluetoothRegisterForAuthenticationEx ERROR_OUTOFMEMORY") << std::endl;
		throw;
	}
	else if (dwResult != ERROR_SUCCESS)
	{
		std::wcerr << _T("BluetoothRegisterForAuthenticationEx failed : ") << WSAGetLastError() << std::endl;
		throw;
	}
}

CBluetoothDeviceManager::~CBluetoothDeviceManager()
{
	BluetoothUnregisterAuthentication(m_hHBLUETOOTH_AUTHENTICATION_REGISTRATION);
}

BLUETOOTH_ADDRESS CBluetoothDeviceManager::GetAddress(void)
{
	return m_BLUETOOTH_DEVICE_INFO_STRUCT.Address;
}

LPCTSTR CBluetoothDeviceManager::GetName(void)
{
	return m_BLUETOOTH_DEVICE_INFO_STRUCT.szName;
}

ULONG CBluetoothDeviceManager::GetClassofDevice(void)
{
	return m_BLUETOOTH_DEVICE_INFO_STRUCT.ulClassofDevice;
}

void CBluetoothDeviceManager::SendPairing(void)
{
	m_BLUETOOTH_DEVICE_INFO_STRUCT.fAuthenticated = 8;
}

void CBluetoothDeviceManager::StartPairing(void)
{
	if (m_BLUETOOTH_DEVICE_INFO_STRUCT.fAuthenticated)
		return;

	DWORD dwResult = BluetoothAuthenticateDeviceEx(m_rCBluetoothRadioManager.GetSafeHwnd(), m_rCBluetoothRadioManager.m_hRadio, &m_BLUETOOTH_DEVICE_INFO_STRUCT, NULL, MITMProtectionNotRequiredBonding);

	if (dwResult == ERROR_SUCCESS)
	{
		std::wcout << _T("BluetoothAuthenticateDeviceEx : ERROR_SUCCESS") << m_BLUETOOTH_DEVICE_INFO_STRUCT.szName << std::endl;

		if (WaitForSingleObject(g_callbackevent, INFINITE) == WAIT_FAILED)
		{
			std::wcerr << _T("WaitForSingleObject call failed with error : ") << GetLastError() << std::endl;
			throw;
		}
		else
		{
			std::wcout << _T("WaitForSingleObject call was successful") << std::endl;
		}
	}
	else if (dwResult == ERROR_CANCELLED)
	{
		std::wcout << _T("BluetoothAuthenticateDeviceEx : The user aborted the operation") << m_BLUETOOTH_DEVICE_INFO_STRUCT.szName << std::endl;
	}
	else if (dwResult == ERROR_INVALID_PARAMETER)
	{
		std::wcout << _T("BluetoothAuthenticateDeviceEx : The device structure specified in pbdti is invalid") << m_BLUETOOTH_DEVICE_INFO_STRUCT.szName << std::endl;
	}
	else if (dwResult == ERROR_NO_MORE_ITEMS)
	{
		std::wcout << _T("BluetoothAuthenticateDeviceEx : The device in pbtdi is already been marked as authenticated") << m_BLUETOOTH_DEVICE_INFO_STRUCT.szName << std::endl;
	}
	else
	{
		std::wcout << _T("BluetoothAuthenticateDeviceEx : ") << WSAGetLastError() << std::endl;
	}
}

BOOL CBluetoothDeviceManager::SocketAttach(SOCKET s)
{
	return m_Socket.Attach(s);
}

BOOL CBluetoothDeviceManager::Attach(const BTH_DEVICE_INFO &rDeviceInfo)
{
	// TRUE == same device
	// FALSE == unkonw or different device

	// address empty
	if ((rDeviceInfo.flags & BDIF_ADDRESS) == FALSE)
		return FALSE;

	// name empty
	if ((rDeviceInfo.flags & BDIF_NAME) == FALSE)
		return FALSE;

	// different classOfDevice
	if (rDeviceInfo.classOfDevice != m_BLUETOOTH_DEVICE_INFO_STRUCT.ulClassofDevice)
		return FALSE;

	// different mac address
	if (rDeviceInfo.address != m_BLUETOOTH_DEVICE_INFO_STRUCT.Address.ullLong)
		return FALSE;

	// different name
	CString strTemp(rDeviceInfo.name);
	if (strTemp.Compare(m_BLUETOOTH_DEVICE_INFO_STRUCT.szName) != 0)
		return FALSE;

	std::wcout << _T("/// after flag start ///") << std::endl;

	std::wcout << _T("BDIF_ADDRESS : ") << (rDeviceInfo.flags&BDIF_ADDRESS) << std::endl;
	std::wcout << _T("BDIF_COD : ") << (rDeviceInfo.flags&BDIF_COD) << std::endl;
	std::wcout << _T("BDIF_NAME : ") << (rDeviceInfo.flags&BDIF_NAME) << std::endl;
	std::wcout << _T("BDIF_PAIRED : ") << (rDeviceInfo.flags&BDIF_PAIRED) << std::endl;
	std::wcout << _T("BDIF_PERSONAL : ") << (rDeviceInfo.flags&BDIF_PERSONAL) << std::endl;
	std::wcout << _T("BDIF_CONNECTED : ") << (rDeviceInfo.flags&BDIF_CONNECTED) << std::endl;
	std::wcout << _T("BDIF_SSP_SUPPORTED : ") << (rDeviceInfo.flags&BDIF_SSP_SUPPORTED) << std::endl;
	std::wcout << _T("BDIF_SSP_PAIRED : ") << (rDeviceInfo.flags&BDIF_SSP_PAIRED) << std::endl;
	std::wcout << _T("BDIF_SSP_MITM_PROTECTED : ") << (rDeviceInfo.flags&BDIF_SSP_MITM_PROTECTED) << std::endl;

	std::wcout << _T("/// after flag end ///") << std::endl;

	std::wcout << _T("GUID_BLUETOOTH_RADIO_IN_RANGE : ") << rDeviceInfo.address << std::endl;
	std::wcout << _T("GUID_BLUETOOTH_RADIO_IN_RANGE : ") << rDeviceInfo.classOfDevice << std::endl;
	std::wcout << _T("GUID_BLUETOOTH_RADIO_IN_RANGE : ") << rDeviceInfo.name << std::endl;

	m_BLUETOOTH_DEVICE_INFO_STRUCT.fAuthenticated = rDeviceInfo.flags & BDIF_PAIRED;
	m_BLUETOOTH_DEVICE_INFO_STRUCT.fConnected = rDeviceInfo.flags & BDIF_CONNECTED;	

	return TRUE;
}

BOOL CBluetoothDeviceManager::Connect(void)
{
	StartPairing();

	if (m_BLUETOOTH_DEVICE_INFO_STRUCT.fAuthenticated == FALSE)
		return FALSE;

	try
	{
		m_Socket.Connect(m_BLUETOOTH_DEVICE_INFO_STRUCT.Address.ullLong);
	}
	catch(...)
	{
		TRACE(_T("try-catch : m_Socket.Connect()"));
		return FALSE;
	}
	
	return m_Socket.IsConnected();
}

void CBluetoothDeviceManager::Close(void)
{
	m_Socket.Close();
}

int CBluetoothDeviceManager::Send(LPVOID pVoid, UINT dLength)
{
	return m_Socket.Send(pVoid, dLength);
}