#include "stdafx.h"

#include "BluetoothSockets.h"

std::shared_ptr<CSocketInitialize> CSocketInitialize::m_pInstance = NULL;

CBluetoothSocketListener::CBluetoothSocketListener(fnc_OnAccept pfnc_OnAccept) :
	m_pfnc_OnAccept(std::move(pfnc_OnAccept))
{
	Listen();
}
CBluetoothSocketListener::~CBluetoothSocketListener(void)
{
	Close();
}

void CBluetoothSocketListener::Init(void)
{
	wchar_t szThisComputerName[MAX_COMPUTERNAME_LENGTH + 1];
	DWORD dwLenComputerName = MAX_COMPUTERNAME_LENGTH + 1;
	if (!GetComputerName(szThisComputerName, &dwLenComputerName))
	{
		std::wcout << _T("GetComputerName is Failed") << std::endl;
		throw;
	}

	m_socket = socket(AF_BTH, SOCK_STREAM, BTHPROTO_RFCOMM);
	if (INVALID_SOCKET == m_socket)
	{
		std::wcout << _T("socket is Failed") << std::endl;
		throw;
	}

	SOCKADDR_BTH SockAddrBthLocal = { 0 };
	SockAddrBthLocal.addressFamily = AF_BTH;
	SockAddrBthLocal.serviceClassId = SerialPortServiceClass_UUID;
	SockAddrBthLocal.port = BT_PORT_ANY;

	if (SOCKET_ERROR == bind(m_socket, (struct sockaddr *) &SockAddrBthLocal, sizeof(SOCKADDR_BTH)))
	{
		std::wcout << _T("bind is Failed") << std::endl;
		throw;
	}

	int iAddrLen = sizeof(SOCKADDR_BTH);
	int ulRetCode = getsockname(m_socket, (struct sockaddr *)&SockAddrBthLocal, &iAddrLen);

	switch (ulRetCode)
	{
	case ERROR_SUCCESS:
		break;
	case WSANOTINITIALISED:
		std::wcout << _T("A successful WSAStartup call must occur before using this API.") << std::endl;
		break;
	case WSAENETDOWN:
		std::wcout << _T("The network subsystem has failed.") << std::endl;
		break;
	case WSAEFAULT:
		std::wcout << _T("The name or the namelen parameter is not a valid part of the user address space, or the namelen parameter is too small.") << std::endl;
		break;
	case WSAEINPROGRESS:
		std::wcout << _T("A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function.") << std::endl;
		break;
	case WSAENOTSOCK:
		std::wcout << _T("The descriptor is not a socket.") << std::endl;
		break;
	case WSAEINVAL:
		std::wcout << _T("The socket has not been bound to an address with bind, or ADDR_ANY is specified in bind but connection has not yet occurred.") << std::endl;
		break;
	default:
		std::wcout << _T("GetSockName is Failed") << std::endl;
		break;
	}

	if (ulRetCode != ERROR_SUCCESS)
		throw;

	CSADDR_INFO csAddrInfo = { NULL, };
	csAddrInfo.LocalAddr.iSockaddrLength = sizeof(SOCKADDR_BTH);
	csAddrInfo.LocalAddr.lpSockaddr = (LPSOCKADDR)&SockAddrBthLocal;
	csAddrInfo.RemoteAddr.iSockaddrLength = sizeof(SOCKADDR_BTH);
	csAddrInfo.RemoteAddr.lpSockaddr = (LPSOCKADDR)&SockAddrBthLocal;
	csAddrInfo.iSocketType = SOCK_STREAM;
	csAddrInfo.iProtocol = BTHPROTO_RFCOMM;

	WSAQUERYSET wsaQuerySet;
	ZeroMemory(&wsaQuerySet, sizeof(WSAQUERYSET));
	wsaQuerySet.dwSize = sizeof(WSAQUERYSET);
	wsaQuerySet.lpServiceClassId = (LPGUID)&SerialPortServiceClass_UUID;

	size_t cbInstanceNameSize = 0;
	HRESULT res = StringCchLength(szThisComputerName, sizeof(szThisComputerName), &cbInstanceNameSize);
	if (FAILED(res))
	{
		std::wcout << _T("StringCchLength is Failed") << std::endl;
		throw;
	}

	cbInstanceNameSize += sizeof(L"Bluetooth Server") + 1;
	wchar_t *pszInstanceName = (LPWSTR)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, cbInstanceNameSize);

	if (NULL == pszInstanceName)
	{
		std::wcout << _T("HeapAlloc is Failed") << std::endl;
		throw;
	}

	StringCbPrintf(pszInstanceName, cbInstanceNameSize, L"%s %s", szThisComputerName, L"Bluetooth Server");

	wsaQuerySet.lpszServiceInstanceName = pszInstanceName;
	wsaQuerySet.lpszComment = L"Example Service instance registered in the directory service through RnR";
	wsaQuerySet.dwNameSpace = NS_BTH;
	wsaQuerySet.dwNumberOfCsAddrs = 1;
	wsaQuerySet.lpcsaBuffer = &csAddrInfo;

	if (SOCKET_ERROR == WSASetService(&wsaQuerySet, RNRSERVICE_REGISTER, 0))
	{
		std::wcout << _T("WSASetService is Failed") << std::endl;

		HeapFree(GetProcessHeap(), HEAP_NO_SERIALIZE, pszInstanceName);

		throw;
	}
	else
	{
		HeapFree(GetProcessHeap(), HEAP_NO_SERIALIZE, pszInstanceName);
	}

	Attach(m_socket);

	if (SOCKET_ERROR == __super::Listen())
	{
		std::wcout << _T("Listen is Failed") << std::endl;
		throw;
	}
}

void CBluetoothSocketListener::OnAccept(int nErrorCode)
{
	// TODO: ⿡ Ưȭ ڵ带 ߰ /Ǵ ⺻ Ŭ ȣմϴ.

	// Error Report
	if (nErrorCode != ERROR_SUCCESS)
	{
		std::wcout << _T("OnAccept is Failed : ") << WSAGetLastError() << std::endl;

		return __super::OnAccept(nErrorCode);
	}

	SOCKET socket = accept(m_socket, NULL, NULL);

	if (socket != INVALID_SOCKET)
	{
		if (m_pfnc_OnAccept(socket) == FALSE)
		{
			closesocket(socket);
		}
	}

	__super::OnAccept(nErrorCode);
}

void CBluetoothSocketListener::Listen(void)
{
	try
	{
		Init();
	}
	catch (...)
	{
	}
}

void CBluetoothSocketListener::Close(void)
{
	if (m_hSocket != INVALID_SOCKET)
		m_socket = Detach();

	closesocket(m_socket);
}

CBluetoothSocket::CBluetoothSocket(void)
{
}

CBluetoothSocket::~CBluetoothSocket(void)
{
	Close();
}

void CBluetoothSocket::OnClose(int nErrorCode)
{
	// TODO: ⿡ Ưȭ ڵ带 ߰ /Ǵ ⺻ Ŭ ȣմϴ.

	// Error Report
	if (nErrorCode != ERROR_SUCCESS)
	{
		m_bIsConnected = FALSE;
		
		std::wcout << _T("OnClose is Failed : ") << WSAGetLastError() << std::endl;

		return __super::OnClose(nErrorCode);
	}

	m_bIsConnected = FALSE;
	
	__super::OnClose(nErrorCode);
}

void CBluetoothSocket::OnOutOfBandData(int nErrorCode)
{
	// TODO: ⿡ Ưȭ ڵ带 ߰ /Ǵ ⺻ Ŭ ȣմϴ.

	// Error Report
	if (nErrorCode != ERROR_SUCCESS)
	{
		std::wcout << _T("OnOutOfBandData is Failed : ") << WSAGetLastError() << std::endl;

		return __super::OnOutOfBandData(nErrorCode);
	}
	
	__super::OnOutOfBandData(nErrorCode);
}

void CBluetoothSocket::OnReceive(int nErrorCode)
{
	// TODO: ⿡ Ưȭ ڵ带 ߰ /Ǵ ⺻ Ŭ ȣմϴ.

	// Error Report
	if (nErrorCode != ERROR_SUCCESS)
	{
		std::wcout << _T("OnReceive is Failed : ") << WSAGetLastError() << std::endl;

		return __super::OnReceive(nErrorCode);
	}

	BYTE buf[4096] = { 0, };
	int cnt = CAsyncSocket::Receive(buf, 4096);

	CString str;
	str.Format(_T("%s"), CA2W((char*)buf, CP_UTF8));

	OutputDebugString(str);

	__super::OnReceive(nErrorCode);
}

void CBluetoothSocket::OnSend(int nErrorCode)
{
	// TODO: ⿡ Ưȭ ڵ带 ߰ /Ǵ ⺻ Ŭ ȣմϴ.

	// Error Report
	if (nErrorCode != ERROR_SUCCESS)
	{
		std::wcout << _T("OnSend is Failed : ") << WSAGetLastError() << std::endl;

		return __super::OnSend(nErrorCode);
	}

	__super::OnSend(nErrorCode);
}

int CBluetoothSocket::Send(LPVOID pVoid, UINT dLength)
{
	return __super::Send(pVoid, dLength);
}

void CBluetoothSocket::Connect(BTH_ADDR RemoteAddr)
{
	if (m_bIsConnected)
		return;

	m_socket = socket(AF_BTH, SOCK_STREAM, BTHPROTO_RFCOMM);
	if (INVALID_SOCKET == m_socket)
	{
		std::wcout << _T("socket is Failed") << std::endl;

		throw;
	}

	SOCKADDR_BTH SockAddrBthServer;

	SockAddrBthServer.addressFamily = AF_BTH;
	SockAddrBthServer.serviceClassId = SerialPortServiceClass_UUID;
	SockAddrBthServer.btAddr = RemoteAddr;
	SockAddrBthServer.port = BT_PORT_ANY;

	int len = sizeof(SockAddrBthServer);

	int nResult = connect(m_socket, (struct sockaddr *)&SockAddrBthServer, sizeof(SockAddrBthServer));
		
	if (nResult == 0)
	{
		// This Line Before Non Async Socket
		// This Line After Async Socket
		if (Attach(m_socket))
		{
			return;
		}
	}

	throw;
}

void CBluetoothSocket::Close(void)
{
	m_bIsConnected = FALSE;
	
	if (m_hSocket != INVALID_SOCKET)
		m_socket = Detach();

	closesocket(m_socket);
}

BOOL CBluetoothSocket::IsConnected(void)
{
	return m_bIsConnected;
}

BOOL CBluetoothSocket::Attach(SOCKET hSocket)
{
	return m_bIsConnected = __super::Attach(hSocket);
}