#pragma once

#include <winsock2.h>
#include <ws2bth.h>
#include <iostream>
#include <strsafe.h>
#include <functional>
#include <memory>

// Server socket

class CSocketInitialize
{
private:
	CSocketInitialize(void) 
	{
		WSADATA WSAData;
		WSAStartup(MAKEWORD(2, 2), &WSAData);
	}

	CSocketInitialize(const CSocketInitialize &rCSocketInitialize) {};

public:
	virtual ~CSocketInitialize(void)
	{
		WSACleanup();
	}

private:
	static std::shared_ptr<CSocketInitialize> m_pInstance;

public:
	static const std::shared_ptr<CSocketInitialize> Get(void)
	{
		if (m_pInstance)
			return m_pInstance;
		else
			return m_pInstance = std::shared_ptr<CSocketInitialize>(new CSocketInitialize());
	}
};

template <class _Ty>
class CSocketInitialized : public _Ty
{
public:
	CSocketInitialized(void) 
	{
		CSocketInitialize::Get();
	}
	virtual ~CSocketInitialized(void)
	{

	}

private:
protected:
public:
};

class CBluetoothSocket;

typedef std::function<BOOL(SOCKET)> fnc_OnAccept;
class CBluetoothSocketListener : private CSocketInitialized<CAsyncSocket>
{
public:
	CBluetoothSocketListener(fnc_OnAccept pfnc_OnAccept);
	virtual ~CBluetoothSocketListener(void);

private:
	SOCKET m_socket;
	fnc_OnAccept m_pfnc_OnAccept;

	void Init(void);
		
protected:
	virtual void OnAccept(int nErrorCode);

public:
	void Listen(void);
	void Close(void);
};

class CBluetoothSocket : private CSocketInitialized<CAsyncSocket>
{
public:
	CBluetoothSocket(void);
	virtual ~CBluetoothSocket(void);

private:
	SOCKET m_socket;

	BOOL m_bIsConnected;
		
	virtual void OnClose(int nErrorCode);
	virtual void OnOutOfBandData(int nErrorCode);
	virtual void OnReceive(int nErrorCode);
	virtual void OnSend(int nErrorCode);

protected:

public:
	int Send(LPVOID pVoid, UINT dLength);

	void Connect(BTH_ADDR RemoteAddr);
	void Close(void);

	BOOL IsConnected(void);

	BOOL Attach(SOCKET hSocket);
};